@@ -6,6 +6,7 @@
 #include "core/file_index_manager.h"
 #include "analyzers/AnythingAnalyzer.h"
 #include "core/config.h"
+#include "utils/string_helper.h"
 
 #include <chrono>
 #include <filesystem>
@@ -21,6 +22,7 @@
 #include "analyzers/chineseanalyzer.h"
 #include "utils/log.h"
 #include "utils/tools.h"
+#include "core/config.h"
 
 #include <glib.h>
 #include <sys/stat.h>  // For statx and struct statx
@@ -69,8 +71,9 @@ file_record make_file_record(const std::filesystem::path& p,
     ret.is_hidden = ret.full_path.find("/.") != std::string::npos;
 
     struct stat statbuf;
-    if (stat(ret.full_path.c_str(), &statbuf) != 0) {
-        spdlog::warn("stat fail: {}", ret.full_path);
+    if (lstat(ret.full_path.c_str(), &statbuf) != 0) {
+        auto err = errno;
+        spdlog::warn("stat fail: {} {}", ret.full_path, strerror(err));
         return ret;
     } else {
         ret.modify_time = statbuf.st_mtim.tv_sec;
@@ -90,6 +93,7 @@ file_record make_file_record(const std::filesystem::path& p,
 }
 
 #define FILE_NAME_FIELD L"file_name"
+#define FILE_NAME_LOWER_FIELD L"file_name_lower"
 #define FULL_PATH_FIELD L"full_path"
 #define FILE_TYPE_FIELD L"file_type"
 #define FILE_EXT_FIELD L"file_ext"
@@ -105,6 +109,9 @@ DocumentPtr create_document(const file_record& record) {
     doc->add(newLucene<Field>(FILE_NAME_FIELD,
         StringUtils::toLower(StringUtils::toUnicode(record.file_name)),
         Field::STORE_YES, Field::INDEX_ANALYZED));
+    doc->add(newLucene<Field>(FILE_NAME_LOWER_FIELD,
+        StringUtils::toLower(StringUtils::toUnicode(record.file_name)),
+        Field::STORE_YES, Field::INDEX_NOT_ANALYZED));
     // Full path with exact match; parser is not needed for deleting and exact searching, which improves perferemce.
     doc->add(newLucene<Field>(FULL_PATH_FIELD,
         StringUtils::toUnicode(record.full_path),
@@ -142,7 +149,7 @@ DocumentPtr create_document(const file_record& record) {
 }
 
 
-#define INDEX_VERSION L"1"
+#define INDEX_VERSION L"2"
 #define INVALID_INDEX_VERSION L"0"
 #define INDEX_VERSION_FIELD L"index_version"
 
@@ -163,6 +170,10 @@ file_index_manager::file_index_manager(const std::string& persistent_index_dir,
                 newLucene<ChineseAnalyzer>(),
                 create, IndexWriter::MaxFieldLengthLIMITED);
             reader_  = IndexReader::open(dir, true);
+            // check if index corrupted by same method
+            if (!commit(index_status::loading)) {
+                throw LuceneException();
+            }
         } catch (const LuceneException& e) {
             spdlog::warn("The index is corrupted: {}", volatile_index_directory_);
             if (writer_) writer_->close();
@@ -178,22 +189,27 @@ file_index_manager::file_index_manager(const std::string& persistent_index_dir,
         searcher_ = newLucene<IndexSearcher>(reader_);
         nrt_searcher_ = newLucene<IndexSearcher>(nrt_reader_);
     } catch (const LuceneException& e) {
-        throw std::runtime_error("Lucene exception: " + StringUtils::toUTF8(e.getError()) +
-            ". Make sure you are running the program as root.");
+        std::string error_msg = "Failed to initialize file_index_manager: " + StringUtils::toUTF8(e.getError());
+        spdlog::critical(error_msg);
+        throw std::runtime_error(error_msg);
     }
 }
 
 file_index_manager::~file_index_manager() {
-    if (writer_) {
-        commit(index_status::closed);
-        writer_->close();
-        persist_index();
-    }
+    try {
+        if (writer_) {
+            commit(index_status::closed);
+            writer_->close();
+            persist_index();
+        }
 
-    if (searcher_) searcher_->close();
-    if (nrt_searcher_) nrt_searcher_->close();
-    if (reader_) reader_->close();
-    if (nrt_reader_) nrt_reader_->close();
+        if (searcher_) searcher_->close();
+        if (nrt_searcher_) nrt_searcher_->close();
+        if (reader_) reader_->close();
+        if (nrt_reader_) nrt_reader_->close();
+    } catch (const LuceneException& e) {
+        spdlog::error("Failed to close file_index_manager: {}", StringUtils::toUTF8(e.getError()));
+    }
 }
 
 void file_index_manager::add_index(const std::string& path) {
@@ -204,7 +220,7 @@ void file_index_manager::add_index(const std::string& path) {
     } catch (const LuceneException& e) {
         spdlog::error("Failed to index {}: {}", path, StringUtils::toUTF8(e.getError()));
     } catch (const std::exception& e) {
-        spdlog::error(e.what());
+        spdlog::error("Failed to index {}: {}", path, e.what());
     }
 }
 
@@ -214,7 +230,9 @@ void file_index_manager::remove_index(const std::string& path) {
         writer_->deleteDocuments(pterm);
         spdlog::debug("Removed index: {}", path);
     } catch (const LuceneException& e) {
-        throw std::runtime_error("Lucene exception: " + StringUtils::toUTF8(e.getError()));
+        spdlog::error("Failed to remove index {}: {}", path, StringUtils::toUTF8(e.getError()));
+    } catch (const std::exception& e) {
+        spdlog::error("Failed to remove index {}: {}", path, e.what());
     }
 }
 
@@ -223,20 +241,37 @@ void file_index_manager::update_index(const std::string& old_path, const std::st
         auto doc = create_document(make_file_record(new_path, pinyin_processor_, file_type_mapping_));
         writer_->updateDocument(newLucene<Term>(FULL_PATH_FIELD, StringUtils::toUnicode(old_path)), doc);
         spdlog::debug("Renamed: {} --> {}", old_path, new_path);
+    } catch (const LuceneException& e) {
+        spdlog::error("Failed to rename index {} to {}: {}", old_path, new_path, StringUtils::toUTF8(e.getError()));
     } catch (const std::exception& e) {
-        spdlog::error(e.what());
+        spdlog::error("Failed to rename index {} to {}: {}", old_path, new_path, e.what());
     }
 }
 
-void file_index_manager::commit(index_status status) {
+bool check_index_corrupted(const std::string& index_directory) {
+    try {
+        FSDirectoryPtr dir = FSDirectory::open(StringUtils::toUnicode(index_directory));
+        Lucene::IndexReaderPtr reader = IndexReader::open(dir, true);
+        reader->close();
+        return false;
+    } catch (const LuceneException& e) {
+        spdlog::error("The index is corrupted: {}, {}", index_directory, StringUtils::toUTF8(e.getError()));
+        return true;
+    }
+}
+
+bool file_index_manager::commit(index_status status) {
     try {
         save_index_status(status);
         set_index_version();
         writer_->commit();
         spdlog::debug("All changes are commited with version: {}", StringUtils::toUTF8(INDEX_VERSION));
     } catch (const LuceneException& e) {
-        spdlog::error("Lucene exception: " + StringUtils::toUTF8(e.getError()));
+        spdlog::error("Failed to commit index: {}", StringUtils::toUTF8(e.getError()));
+        return false;
     }
+
+    return !check_index_corrupted(volatile_index_directory_);
 }
 
 void file_index_manager::persist_index() {
@@ -260,45 +295,50 @@ void file_index_manager::persist_index() {
     spdlog::debug("Persist index to {}", persistent_index_directory_);
 }
 
-bool file_index_manager::indexed() const {
-    return document_size() > 0;
-}
+// bool file_index_manager::indexed() const {
+//     return document_size() > 0;
+// }
 
-void file_index_manager::test(const String& path) {
-    spdlog::info("test path: {}", StringUtils::toUTF8(path));
-    // AnalyzerPtr analyzer = newLucene<StandardAnalyzer>(LuceneVersion::LUCENE_CURRENT);
-    AnalyzerPtr analyzer = newLucene<AnythingAnalyzer>();
+// void file_index_manager::test(const String& path) {
+//     spdlog::info("test path: {}", StringUtils::toUTF8(path));
+//     // AnalyzerPtr analyzer = newLucene<StandardAnalyzer>(LuceneVersion::LUCENE_CURRENT);
+//     AnalyzerPtr analyzer = newLucene<AnythingAnalyzer>();
     
-    // Use a StringReader to simulate input
-    TokenStreamPtr tokenStream = analyzer->tokenStream(L"", newLucene<StringReader>(path));
-    // TokenPtr token = newLucene<Token>();
+//     // Use a StringReader to simulate input
+//     TokenStreamPtr tokenStream = analyzer->tokenStream(L"", newLucene<StringReader>(path));
+//     // TokenPtr token = newLucene<Token>();
     
-    // Tokenize and print out the results
-    while (tokenStream->incrementToken()) {
-        spdlog::info("Token: {}", StringUtils::toUTF8(tokenStream->toString()));
-    }
-}
+//     // Tokenize and print out the results
+//     while (tokenStream->incrementToken()) {
+//         spdlog::info("Token: {}", StringUtils::toUTF8(tokenStream->toString()));
+//     }
+// }
 
-void file_index_manager::pinyin_test(const std::string& path) {
-    spdlog::info("pinyin test path: {}", path);
-    auto pinyin_path = pinyin_processor_.convert_to_pinyin(path);
-    test(StringUtils::toUnicode(pinyin_path));
-}
+// void file_index_manager::pinyin_test(const std::string& path) {
+//     spdlog::info("pinyin test path: {}", path);
+//     auto pinyin_path = pinyin_processor_.convert_to_pinyin(path);
+//     test(StringUtils::toUnicode(pinyin_path));
+// }
 
-int32_t file_index_manager::document_size(bool nrt) const {
-    return nrt ? nrt_reader_->numDocs() : reader_->numDocs();
-}
+// int32_t file_index_manager::document_size(bool nrt) const {
+//     return nrt ? nrt_reader_->numDocs() : reader_->numDocs();
+// }
 
 std::string file_index_manager::index_directory() const {
     return persistent_index_directory_;
 }
 
-QStringList file_index_manager::traverse_directory(const QString& path, bool nrt) {
-    if (path.isEmpty()) {
+std::vector<std::string> file_index_manager::traverse_directory(const std::string& path, bool nrt) {
+    if (path.empty()) {
         return {};
     }
 
-    String query_terms = StringUtils::toUnicode(path.toStdString());
+    std::string path_with_slash = path;
+    if (!string_helper::ends_with(path_with_slash, "/")) {
+        path_with_slash += "/";
+    }
+
+    String query_terms = StringUtils::toUnicode(path_with_slash);
 
     try {
         SearcherPtr searcher;
@@ -316,16 +356,16 @@ QStringList file_index_manager::traverse_directory(const QString& path, bool nrt
         auto query = newLucene<PrefixQuery>(newLucene<Term>(FULL_PATH_FIELD, query_terms));
         auto search_results = searcher->search(query, max_results);
 
-        QStringList results;
+        std::vector<std::string> results;
         results.reserve(search_results->scoreDocs.size());
         for (const auto& score_doc : search_results->scoreDocs) {
             DocumentPtr doc = searcher->doc(score_doc->doc);
-            results.append(QString::fromStdWString(doc->get(FULL_PATH_FIELD)));
+            results.push_back(StringUtils::toUTF8(doc->get(FULL_PATH_FIELD)));
         }
 
         return results;
     } catch (const LuceneException& e) {
-        spdlog::error("Lucene exception: " + StringUtils::toUTF8(e.getError()));
+        spdlog::error("Failed to traverse directory {}: {}", path, StringUtils::toUTF8(e.getError()));
         return {};
     }
 }
@@ -344,32 +384,38 @@ bool file_index_manager::document_exists(const std::string &path, bool only_chec
 
 bool file_index_manager::refresh_indexes(const std::vector<std::string>& blacklist_paths) {
     bool index_changed = false;
-    std::error_code ec;
-    spdlog::info("Refreshing file indexes...");
-    try_refresh_reader();
-    auto query = newLucene<Lucene::MatchAllDocsQuery>();
-    auto num_docs = reader_->numDocs();
-    if (num_docs > 0) {
-        auto search_results = searcher_->search(query, num_docs);
-        std::vector<std::string> remove_list;
-        for (const auto& score_doc : search_results->scoreDocs) {
-            DocumentPtr doc = searcher_->doc(score_doc->doc);
-            std::filesystem::path full_path(doc->get(FULL_PATH_FIELD));
-            if (full_path.empty()) {
-                // doc is metadata, not a file
-                continue;
+    try {
+        std::error_code ec;
+        spdlog::info("Refreshing file indexes...");
+        try_refresh_reader();
+        auto query = newLucene<Lucene::MatchAllDocsQuery>();
+        auto num_docs = reader_->numDocs();
+        if (num_docs > 0) {
+            auto search_results = searcher_->search(query, num_docs);
+            std::vector<std::string> remove_list;
+            for (const auto& score_doc : search_results->scoreDocs) {
+                DocumentPtr doc = searcher_->doc(score_doc->doc);
+                std::filesystem::path full_path(doc->get(FULL_PATH_FIELD));
+                if (full_path.empty()) {
+                    // doc is metadata, not a file
+                    continue;
+                }
+                if (!std::filesystem::exists(full_path, ec) ||
+                    is_path_in_blacklist(full_path.string(), blacklist_paths)) {
+                    remove_list.push_back(full_path.string());
+                }
             }
-            if (!std::filesystem::exists(full_path, ec) ||
-                is_path_in_blacklist(full_path.string(), blacklist_paths)) {
-                remove_list.push_back(full_path.string());
-            }
-        }
 
-        for (const auto& path : remove_list) {
-            // Remove non-existent path from the indexes
-            remove_index(path);
-            index_changed = true;
+            for (const auto& path : remove_list) {
+                // Remove non-existent path from the indexes
+                remove_index(path);
+                index_changed = true;
+            }
         }
+    } catch (const LuceneException& e) {
+        spdlog::error("Failed to refresh indexes: {}", StringUtils::toUTF8(e.getError()));
+    } catch (const std::exception& e) {
+        spdlog::error("Failed to refresh indexes: {}", e.what());
     }
 
     return index_changed;
@@ -377,11 +423,17 @@ bool file_index_manager::refresh_indexes(const std::vector<std::string>& blackli
 
 void file_index_manager::set_index_invalid()
 {
-    set_index_version();
+    try {
+        set_index_version();
 
-    DocumentPtr doc = newLucene<Document>();
-    doc->add(newLucene<Field>(INDEX_VERSION_FIELD, INVALID_INDEX_VERSION, Field::STORE_YES, Field::INDEX_NOT_ANALYZED));
-    writer_->updateDocument(newLucene<Term>(INDEX_VERSION_FIELD, INDEX_VERSION), doc);
+        DocumentPtr doc = newLucene<Document>();
+        doc->add(newLucene<Field>(INDEX_VERSION_FIELD, INVALID_INDEX_VERSION, Field::STORE_YES, Field::INDEX_NOT_ANALYZED));
+        writer_->updateDocument(newLucene<Term>(INDEX_VERSION_FIELD, INDEX_VERSION), doc);
+    } catch (const LuceneException& e) {
+        spdlog::error("Failed to set index invalid: {}", StringUtils::toUTF8(e.getError()));
+    } catch (const std::exception& e) {
+        spdlog::error("Failed to set index invalid: {}", e.what());
+    }
 }
 
 void file_index_manager::try_refresh_reader(bool nrt) {
@@ -409,13 +461,19 @@ void file_index_manager::try_refresh_reader(bool nrt) {
 
 void file_index_manager::prepare_index() {
     spdlog::info("Preparing index...");
-    if (!std::filesystem::exists(volatile_index_directory_)) {
-        if (!std::filesystem::exists(persistent_index_directory_)) {
+    std::error_code ec;
+    if (!std::filesystem::exists(volatile_index_directory_, ec)) {
+        if (ec) {
+            spdlog::error("Failed to check volatile index directory: {}", ec.message());
+            exit(APP_RESTART_CODE);
+        }
+
+        if (!std::filesystem::exists(persistent_index_directory_, ec)) {
             spdlog::info("Persistent index directory does not exist: {}", persistent_index_directory_);
             return;
         }
 
-        std::error_code ec;
+        ec.clear();
         std::filesystem::copy(persistent_index_directory_,
                               volatile_index_directory_,
                               std::filesystem::copy_options::recursive,
@@ -434,7 +492,8 @@ void file_index_manager::check_index_version() {
     spdlog::info("Checking index version...");
     Lucene::IndexReaderPtr reader;
 
-    if (!std::filesystem::exists(volatile_index_directory_)) {
+    std::error_code ec;
+    if (!std::filesystem::exists(volatile_index_directory_, ec)) {
         spdlog::info("Index directory does not exist: {}", volatile_index_directory_);
         return;
     }
@@ -452,12 +511,19 @@ void file_index_manager::check_index_version() {
     }
 
     // 查找数据库版本
-    TermPtr term = newLucene<Term>(INDEX_VERSION_FIELD, INDEX_VERSION);
-    QueryPtr query = newLucene<TermQuery>(term);
-    Lucene::SearcherPtr searcher = newLucene<IndexSearcher>(reader);
-    Lucene::TopDocsPtr top_docs = searcher->search(query, 1);
-    bool found = top_docs->scoreDocs.size() == 1;
-    reader->close();
+    bool found = false;
+    try {
+        TermPtr term = newLucene<Term>(INDEX_VERSION_FIELD, INDEX_VERSION);
+        QueryPtr query = newLucene<TermQuery>(term);
+        Lucene::SearcherPtr searcher = newLucene<IndexSearcher>(reader);
+        Lucene::TopDocsPtr top_docs = searcher->search(query, 1);
+        found = top_docs->scoreDocs.size() == 1;
+        reader->close();
+    } catch (const LuceneException& e) {
+        spdlog::error("Failed to check index version: {}", StringUtils::toUTF8(e.getError()));
+    } catch (const std::exception& e) {
+        spdlog::error("Failed to check index version: {}", e.what());
+    }
     if (found) {
         spdlog::info("The index version is expected({}): {}", StringUtils::toUTF8(INDEX_VERSION), volatile_index_directory_);
     } else {
@@ -471,9 +537,15 @@ void file_index_manager::set_index_version() {
     static std::once_flag flag;
     std::call_once(flag, [this]() {
         // 保存版本号到数据库
-        DocumentPtr doc = newLucene<Document>();
-        doc->add(newLucene<Field>(INDEX_VERSION_FIELD, INDEX_VERSION, Field::STORE_YES, Field::INDEX_NOT_ANALYZED));
-        writer_->updateDocument(newLucene<Term>(INDEX_VERSION_FIELD, INDEX_VERSION), doc);
+        try {
+            DocumentPtr doc = newLucene<Document>();
+            doc->add(newLucene<Field>(INDEX_VERSION_FIELD, INDEX_VERSION, Field::STORE_YES, Field::INDEX_NOT_ANALYZED));
+            writer_->updateDocument(newLucene<Term>(INDEX_VERSION_FIELD, INDEX_VERSION), doc);
+        } catch (const LuceneException& e) {
+            spdlog::error("Failed to set index version: {}", StringUtils::toUTF8(e.getError()));
+        } catch (const std::exception& e) {
+            spdlog::error("Failed to set index version: {}", e.what());
+        }
     });
 }
 
@@ -485,10 +557,8 @@ static const char * const status_json_template = R"({
 )";
 
 void file_index_manager::save_index_status(index_status status) {
-    auto now = std::chrono::system_clock::now();
-    auto now_time_t = std::chrono::system_clock::to_time_t(now);
-    std::stringstream ss;
-    ss << std::put_time(std::localtime(&now_time_t), "%Y-%m-%dT%H:%M:%S");
+    g_autoptr(GDateTime) now = g_date_time_new_now_local();
+    g_autofree char *time_str = g_date_time_format(now, "%Y-%m-%dT%H:%M:%S");
 
     std::string status_str;
     switch (status) {
@@ -511,7 +581,7 @@ void file_index_manager::save_index_status(index_status status) {
 
     char status_json[1024];
     int written = snprintf(status_json, sizeof(status_json), status_json_template,
-             ss.str().c_str(),
+             time_str,
              status_str.c_str(),
              StringUtils::toUTF8(INDEX_VERSION).c_str());
     if (written < 0 || written >= (int)sizeof(status_json)) {
@@ -519,9 +589,13 @@ void file_index_manager::save_index_status(index_status status) {
         return;
     }
 
-    std::ofstream status_file(volatile_index_directory_ + "/status.json");
-    status_file << status_json;
-    status_file.close();
+    try {
+        std::ofstream status_file(volatile_index_directory_ + "/status.json");
+        status_file << status_json;
+        status_file.close();
+    } catch (const std::exception& e) {
+        spdlog::error("Failed to save index status: {}", e.what());
+    }
 }
 
 ANYTHING_NAMESPACE_END
